package cse403.observer;

import java.util.ArrayList;

abstract aspect ObserverPattern
{
    abstract pointcut stateChanges(Subject subject);

    after(Subject subject): stateChanges(subject) 
    {
        for (Observer observer: subject.getObservers())
            observer.update();
    }

    /*
     * Code that is declared for an implementation of a Subject.java.
     */
    private ArrayList <Observer> Subject.observers = new ArrayList <Observer>();

    public void Subject.addObserver(Observer observer)
    {
        observers.add(observer);
        observer.setSubject(this);
    }

    public void Subject.remove(Observer observer)
    {
        observers.remove(observers);
        observer.setSubject(null);
    }

    public ArrayList <Observer> Subject.getObservers()
    {
        return observers;
    }

    /*
     * Code that is declared for an implementation of an Observer.java
     */
    private Subject Observer.subject = null;

    public void Observer.setSubject(Subject aSubject)
    {
        subject = aSubject;
    }

    public Subject Observer.getSubject()
    {
        return subject;
    }
}
